#include "ClientSocket.h"

#ifndef _WIN32
#include <netdb.h>
#include <arpa/inet.h>
#include <sys/errno.h>
#endif

ClientSocket::ClientSocket( const _TCHAR* host, int port )
{
	m_port = port;
	m_szHostName = host;
}

ClientSocket::~ClientSocket()
{
}

const _TCHAR* ClientSocket::getRemoteHost( void ) const
{
	return m_szHostName.c_str();
}

bool ClientSocket::makeSocket( unsigned long uTimeout )
{	
	struct sockaddr_in socketInfo;
	struct hostent *hostinfo;
	
	SOCKET sock = socket( AF_INET, SOCK_STREAM, 0 );
	if( sock == INVALID_SOCKET )
	{
		return false;
	}
#ifdef _WIN32
	int noDelay = 1;
	if( setsockopt( sock, IPPROTO_TCP, TCP_NODELAY, (const char* )&noDelay, sizeof( noDelay ) ) != 0 )
	{
		ClSocket sk( sock );
		sk.setStatus( -1 );
		return false;
	}
#endif
	socketInfo.sin_family = AF_INET;
	socketInfo.sin_port = htons( m_port );

#ifdef _UNICODE
	char* host = new char[m_hostName->getElements() + 1];
	m_hostName->getString( host );
	hostinfo = gethostbyname( host );
	if( hostinfo == NULL )
	{
		socketInfo.sin_addr.s_addr = inet_addr( host );
	}
	else
	{
		socketInfo.sin_addr = *(struct in_addr*)hostinfo->h_addr;
	}
	delete host;
#else
	hostinfo = gethostbyname( m_szHostName.c_str() );
	if( hostinfo == NULL )
	{
		socketInfo.sin_addr.s_addr = inet_addr( m_szHostName.c_str() );
	}
	else
	{
		socketInfo.sin_addr = *(struct in_addr*)hostinfo->h_addr;
	}
#endif

	if ( uTimeout != 0 ) {
		setBlocking( sock, false );
	}

	int status = connect( sock, (struct sockaddr*)&socketInfo, sizeof( struct sockaddr_in ) );
	if( status == SOCKET_ERROR )
	{
#ifdef _WIN32
		if ( !m_bBlocking && WSAEWOULDBLOCK == WSAGetLastError() )
#else
		if ( !m_bBlocking && ( EAGAIN == errno || EINPROGRESS == errno ) )
#endif
		{
			timeval st;
			st.tv_sec = uTimeout/1000000;
			st.tv_usec = uTimeout%1000000;

			fd_set wsfd;
			FD_ZERO( &wsfd );

			fd_set esfd;
			FD_ZERO( &esfd );

			FD_SET( sock, &wsfd );
			FD_SET( sock, &esfd );

			int iSelect = select( sock + 1, 0, &wsfd, &esfd, &st );
			if (0==iSelect || SOCKET_ERROR==iSelect || FD_ISSET( sock, &esfd ))
			{
				if ( uTimeout != 0 ) {
					setBlocking( sock, true );
				}

				ClSocket sk( sock );
				sk.setStatus( -1 );
				return false;
			}
		}
		else
		{
			if ( uTimeout != 0 ) {
				setBlocking( sock, true );
			}
			ClSocket sk( sock );
			sk.setStatus( -1 );
			return false;
		}
 	}

	if ( uTimeout != 0 ) {
		setBlocking( sock, true );
	}

	setSocket( sock );
	return true;
}
