#include "Socket.h"

#ifdef _WIN32
	#define MSG_NOSIGNAL	0
	#define	MSG_WAITALL		0
#else
	#include <sys/errno.h>

	#ifdef _DARWIN
		#define MSG_NOSIGNAL	0
	#endif
#endif

#ifdef _WIN32
#else
    #include <netdb.h>
    #include <sys/socket.h>
    #include <netinet/in.h>
    #include <arpa/inet.h>
#endif

bool Socket::m_initialized = false;

Socket::Socket()
{
	m_socket = INVALID_SOCKET;
	m_socketStatus = 0;
}

Socket::Socket( SOCKET sock)
{
	setSocket( sock );
}

Socket::~Socket()
{
	setStatus( -1 );
}

int Socket::getData( char *buffer, int bufferSize )
{
	int readed, total = 0;

	while( total < bufferSize )
	{
		if( (readed = getStatus()) < 0 )
		{
			return readed;
		}

		readed = recv( m_socket, &buffer[total], bufferSize - total, MSG_NOSIGNAL|MSG_WAITALL );

		if( (readed == 0) || (readed == SOCKET_ERROR) )
		{
#ifdef _WIN32
			if( (readed == SOCKET_ERROR) && (WSAGetLastError () == WSAEWOULDBLOCK) )
#else			
			if( (readed == SOCKET_ERROR) && (errno == EAGAIN) )
#endif
			{
			    readed = 0;
			    Sleep( 10 );
			}
			else
			{
			    setStatus( -1 );
			    return -1;
			}
		}
		total += readed;
	}
	return total;
}

int Socket::getLine( _TCHAR* buffer, int bufferSize )
{
	int i = 0, readed;
	_TCHAR c;

	while( i < bufferSize )
	{
		readed = getData( (char*)&c, sizeof( _TCHAR ) );
		if( readed < 0 )
		{
			buffer[0] = _T('\0');
			return readed;
		}
		if( (c == _T('\r')) || (c == _T('\n')) || (c == _T('\0')) )
		{
			break;
		}
		buffer[i] = c;
		i++;
	}
	buffer[i] = _T('\0');
	return i;
}

int Socket::getStatus( void )
{
	if( m_socket== INVALID_SOCKET )
	{
		return -1;
	}
	return m_socketStatus;
}

int Socket::sendData( const char *buffer, int bufferSize )
{
	int sended, total = 0;			
	if( bufferSize == 0 )
	{
		return getStatus();
	}

	while( total < bufferSize )
	{
		if( (sended = getStatus ()) < 0 )
		{
			return sended;
		}
		sended = send( m_socket, &buffer[total], bufferSize - total, MSG_NOSIGNAL );
		if( (sended == 0) || (sended == SOCKET_ERROR) )
		{
#ifdef _WIN32
			if( (sended == SOCKET_ERROR) && (WSAGetLastError () == WSAEWOULDBLOCK) )
#else			
			if( (sended == SOCKET_ERROR) && (errno == EAGAIN) )
#endif
			{
			    sended = 0;
			    Sleep( 10 );
			}
			else
			{
			    setStatus( -1 );
			    return -1;
			}
		}
		total += sended;
	}
	return total;
}

int Socket::sendLine( const _TCHAR* buffer, int buffSize )
{
	return sendData( (const char* )(buffer), buffSize*sizeof( _TCHAR ) );
}

void Socket::setStatus( int status )
{
	if( m_socketStatus < 0 )
	{
		return;
	}
	m_socketStatus = status;

	if( (status < 0) && (m_socket != INVALID_SOCKET) )
	{

#ifdef _WIN32
		closesocket( m_socket );
#else
		shutdown( m_socket, SHUT_RDWR );
		close( m_socket );
#endif
		m_socket = INVALID_SOCKET;
	}
}

void Socket::setSocket( SOCKET sock )
{
	m_socket = sock;
	m_socketStatus = 0;
#if defined(_DARWIN)
        int p=1;
        setsockopt(m_socket, SOL_SOCKET, SO_NOSIGPIPE,(const char*)&p, sizeof(int));
#endif
}

bool Socket::initSocketLibrary( void )
{
#ifdef _WIN32

	if( !m_initialized )
	{
		WORD socketVersionRequested;
		WSADATA wsaData;
		socketVersionRequested = MAKEWORD (2,0);
		if (WSAStartup (socketVersionRequested, &wsaData) != 0)
		{ 
			WSACleanup();
		}
		else
		{
			m_initialized = true;
		}
	}
#else
	m_initialized = true;
#endif

	return m_initialized;
}

void Socket::closeSocketLibrary( void )
{
#ifdef _WIN32
	WSACleanup();
#endif
	m_initialized = false;
}
