
#include "SysThread.h"
#include <limits.h>

#ifndef _WIN32
#define	INVALID_HANDLE_VALUE	0
#endif

#include "config.h"

const unsigned int SysThread::INFINITE_WAIT = UINT_MAX;

SysThread::SysThread( void )
{
	m_bRunning = false;
	m_bExited = true;
	m_thread = INVALID_HANDLE_VALUE;
}

SysThread::~SysThread()
{
}

#ifdef _WIN32
DWORD WINAPI runStub( LPVOID mthread )
#else
void* runStub( void* mthread)
#endif
{
	SysThread* pThread = static_cast<SysThread* >(mthread);
	pThread->m_bRunning = true;
	pThread->run();
	pThread->m_bRunning = false;
	pThread->m_bExited = true;
#ifdef _WIN32
	return 0;
#else
	return NULL;
#endif
}
#define SYSTHREAD_STACK_SIZE 4096
bool SysThread::start( void )
{
	if( m_bExited )
	{
		m_bExited = false;
#ifdef _WIN32
		DWORD dw;
		if( (m_thread = CreateThread( NULL, 4096, runStub, this, 0, &dw )) == INVALID_HANDLE_VALUE )
		{
			m_bRunning = false;
			m_bExited = true;
			return false;
		}
#else
		pthread_attr_t attr;
		pthread_attr_init( &attr );
		pthread_attr_setdetachstate( &attr, PTHREAD_CREATE_DETACHED );
		//pthread_attr_setstacksize(&attr, (SYSTHREAD_STACK_SIZE > PTHREAD_STACK_MIN) ? SYSTHREAD_STACK_SIZE : PTHREAD_STACK_MIN);
		pthread_attr_setstacksize( &attr, 32*1024 );
		pthread_attr_setschedpolicy( &attr, SCHED_OTHER );
		pthread_attr_setscope( &attr, PTHREAD_SCOPE_SYSTEM );
		if( pthread_create( &m_thread, &attr, runStub, this ) != 0 )
		{
			m_thread = INVALID_HANDLE_VALUE;
			m_bRunning = false;
			m_bExited = true;
			return false;
		}
#endif
	}
	return true;
}

bool SysThread::stop( unsigned int timeout )
{
	m_bRunning = false;
	if( !m_bExited )
	{
		for( unsigned int i = 0; (i <= timeout/100) || (timeout == INFINITE_WAIT); i++)
		{
			m_bRunning = false;
			if( m_bExited )
			{
				break;
			}
			Sleep( 10 );
		}
	}
#ifdef _WIN32
	if( m_thread != INVALID_HANDLE_VALUE )
	{
		CloseHandle( m_thread );
		m_thread = INVALID_HANDLE_VALUE;
	}
#else
	m_thread = INVALID_HANDLE_VALUE;
#endif
	return m_bExited;
}

void SysThread::run( void )
{
}
